//
// Commander module for the command processing functions
//
// This software demonstrates some principles in receiving text
// from a PC and interpreting that text as commands and numbers.

// Clear out the message array in preparation for receiving
// a new message.
void initializeMessageArray()
{
   // Fill with the terminator character
   for(int i = 0; i < MAX_MESSAGE; i++)
  {    
    // Fill array with the terminal character
    messageArray[i] = TERMINATOR;
  }   
}

// Load a message from the PC until receiving the message 
// terminator symbol, then call the message parsing function.
void loadMessageArray()
{
  //choke = 1;
  //Serial.println("Enter loadMessageArray()");
  if(messageArrayCount <= MAX_MESSAGE)
  { 
    // put the character in the array
    messageArray[messageArrayCount] = Serial.read();
  }
  
  if(messageArray[messageArrayCount] == TERMINATOR)
  {   
    parseArray();  
  }
  else messageArrayCount++;
  //Serial.println("leave loadMessageArray()");
  //choke = 0;
}

// Parse the message array for a valid message. If a valid 
// message is present, then parse the message as a command.
void parseArray()
{    
  int i = 0;
  int j = 0;
  while( (messageArray[i] != TERMINATOR) && (i < MAX_MESSAGE) )
  {
    while( (messageArray[i] != SEPARATOR) && (messageArray[i] != TERMINATOR))
    {
      commandArray[j++] = messageArray[i++]; 
    } 
    commandArrayCount = j;
    parseCommand();
    j = 0;
    i++;
  }
  messageArrayCount=0;
  initializeMessageArray(); 

  // Clear the receive buffer before going back to the loop()
  while(Serial.available()) Serial.read(); 
}

// Parse a command from the PC and call the function
// associated with the command
void parseCommand()
{
  switch (commandArray[0])
  {
     case 'A': // Set Alarm # 1 to 5
       ACase();
       break;  
      case 'B': // AM or PM or 24 hr
       BCase();
       break;    
     case 'D': // Day
       DCase();
       break;     
      case 'H': // Hour
       HCase();
       break;
      case 'M': // Minute (MI) or Month (MO)
       MCase();
       break;
      case 'S': // Second
       SCase();
       break;
      case 'Y': // Year
       YCase();
       break;    
      case 'N': // show NOW
       NCase();
       break;      
      case 'P': // Use PC to set date time
       PCase();
       break;   
      case 'L': // Show all alarms
       LCase();
       break;  
      case 'U': // Show all alarms
       UCase();
       break;  
      case 'I': // Hour
       ICase();
       break;  
      case 'T': // Set Alarm Type
       TCase();
       break;  
      case 't': // Test cases
       tCase();
       break; 
      case 'C': // calibrate
       CCase();
       break;        
     default:
       Serial.print("parseCommand Bad commandArray[0] = ");
       Serial.println(commandArray[0]);
       break;  
  }
#if defined(DEBUG)   
  //Serial.println("Exit parseCommand"); 
#endif  
    
}

//
// Functions associated with cases
//

// Calibrate
void CCase(){
#if defined(DEBUG)   
  Serial.println("CCase"); 
#endif 
  uint8_t val1,val2,val3,val4;

  // Get the four bytes for the 32-bit Unix time
  val1 = commandArray[1];
  val2 = commandArray[2];
  val3 = commandArray[3];  
  val4 = commandArray[4];  
  
  // Get the add or subtract variable
  addsubtract = commandArray[5];  
  
  // Create the 32-bit calibration value from the 4 bytes
  calibration = (uint32_t)val1 + \
        ((uint32_t)(val2)<<8) + \
        ((uint32_t)(val3)<<16) + \
        ((uint32_t)(val4)<<24);  
  
  // Use 0 to turn it off the calibration alarm
  if(calibration == 0){
    cal_is_set = 0; // Calibration is no longer active
    Serial.println("Stop calibration");
    return;
  }
  
  // Set calibration to active
  cal_is_set = 1; 
  
  // Get the date and time from the DS1307
  DateTime now = RTC.now(); 
  // Load this into the initial cal_start_time variable
  cal_start_time = now.unixtime();
  
  // Save values to EEPROM
  write_calibration();
  
  // Inform the PC   
  Serial.print("calibration = ");
  Serial.println(calibration);   
  Serial.print("addsubtract = ");
  Serial.println(addsubtract);
}

// Test cases
void tCase(){
#if defined(DEBUG)   
  Serial.println("tCase"); 
#endif 
  switch (commandArray[1])
  {
     case '1': 
       t1Case();
       break; 
     case '2':
       t2Case();
       break;      
     default:
       Serial.print("TCase Bad commandArray[1] = ");
       Serial.println(commandArray[1]);
       break; 
  } 
}

// Test1 case
void t1Case(){
#if defined(DEBUG)   
  Serial.println("t1Case"); 
#endif 

  Serial.print("Alarm 1 in EEPROM: ");
  read_alarm(1);
  Serial.print("alarm1_datetime = ");
  Serial.println(alarm1.datetime.unixtime());
  Serial.print("alarm1_type = ");
  Serial.println(alarm1.type);
  Serial.print("alarm1_set = ");
  Serial.println(alarm1.is_set);
  Serial.print("alarm1_tripped = ");
  Serial.println(alarm1.is_tripped);

/* Serial.print("Alarm 2 in EEPROM: ");
  read_alarm(2);
  Serial.print("alarm2_datetime = ");
  Serial.println(alarm2_datetime);  
  Serial.print("alarm2_type = ");
  Serial.println(alarm2_type);
  Serial.print("alarm2_set = ");
  Serial.println(alarm2_set);
  Serial.print("alarm2_tripped = ");
  Serial.println(alarm2_tripped);
  
  Serial.print("Alarm 3 in EEPROM: ");
  read_alarm(3);
  Serial.print("alarm3_datetime = ");
  Serial.println(alarm3_datetime);  
  Serial.print("alarm3_type = ");
  Serial.println(alarm3_type);
  Serial.print("alarm3_set = ");
  Serial.println(alarm3_set);
  Serial.print("alarm3_tripped = ");
  Serial.println(alarm3_tripped);
  
  Serial.print("Alarm 4 in EEPROM: ");
  read_alarm(4);  
  Serial.print("alarm4_datetime = ");
  Serial.println(alarm4_datetime);
  Serial.print("alarm4_type = ");
  Serial.println(alarm4_type);
  Serial.print("alarm4_set = ");
  Serial.println(alarm4_set);
  Serial.print("alarm4_tripped = ");
  Serial.println(alarm4_tripped);
  
  Serial.print("Alarm 5 in EEPROM: "); 
  read_alarm(5);  
  Serial.print("alarm5_datetime = ");
  Serial.println(alarm5_datetime);
  Serial.print("alarm5_type = ");
  Serial.println(alarm5_type);
  Serial.print("alarm5_set = ");
  Serial.println(alarm5_set);
  Serial.print("alarm5_tripped = ");
  Serial.println(alarm5_tripped);
*/  
}
// Test1 case
void t2Case(){
#if defined(DEBUG)   
  Serial.println("t2Case"); 
#endif 
  Serial.println("Alarms in EEPROM set to 0: ");

  alarm1.datetime = 0;
  alarm1.type = 0;
  alarm1.is_set = 0;
  alarm1.is_tripped = 0; 
  write_alarm(1,alarm1);//,0L,0,0,0);  
  /*write_alarm(2,0L,0,0,0);
  write_alarm(3,0L,0,0,0);
  write_alarm(4,0L,0,0,0);
  write_alarm(5,0L,0,0,0);*/
}


// Is the RTC running
void ICase(){
#if defined(DEBUG)   
  Serial.println("ICase"); 
#endif 

  if(!RTC.isrunning()) Serial.println("N");  
  else Serial.println("Y"); 
}

// Set Alarm # 1 to 9
void ACase(){
#if defined(DEBUG)   
  Serial.println("ACase");
#endif 
  switch (commandArray[1])
  {
     case '1': // Alarm1
       setAlarm(alarm1.datetime);
       alarm1.is_set = 1;
       alarm1.is_tripped = 0;
       alarm1.type = commandArray[14]-48;
       show_date("A1", alarm1.datetime);
       write_alarm(1,alarm1);
       break; 
     case '2': // Alarm2
       setAlarm(alarm2.datetime);
       alarm2.is_set = 1;
       alarm2.is_tripped = 0;
       alarm2.type = commandArray[14];
       show_date("A2", alarm2.datetime);
       write_alarm(2,alarm2);
       break;
      case '3': // Alarm3
       setAlarm(alarm3.datetime);
       alarm3.is_set = 1;
       alarm3.is_tripped = 0;
       alarm3.type = commandArray[14];
       show_date("A3", alarm3.datetime);
       write_alarm(3,alarm3);
       break;     
      case '4': // Alarm4
       setAlarm(alarm4.datetime);
       alarm4.is_set = 1;
       alarm5.is_tripped = 0;
       alarm4.type = commandArray[14];
       show_date("A4", alarm4.datetime);
       write_alarm(4,alarm4);
       break;      
      case '5': // Alarm
       setAlarm(alarm5.datetime);
       alarm5.is_set = 1;
       alarm5.is_tripped = 0;
       alarm5.type = commandArray[14];
       show_date("A5", alarm5.datetime);
       write_alarm(5,alarm5);
       break;            
     default:
       Serial.println("ACase Bad commandArray[1].");
       break;   
  }  
#if defined(DEBUG)   
  //Serial.println("Exit ACase"); 
#endif  
}

// AM or PM or 24 hr
void BCase(){
#if defined(DEBUG)   
  Serial.println("BCase"); 
#endif 
}

 // Day
void DCase(){
#if defined(DEBUG)   
  //Serial.println("DCase"); 
#endif 

  if( (commandArray[1] == 'a') && \
    (commandArray[2] == 't') &&     \
    (commandArray[3] == 'e') ) show_date();
  else  Serial.println("DCase Bad commandArray[].");  

}

// Hour
void HCase(){
#if defined(DEBUG)
  Serial.println("HCase"); 
 #endif 
}

// Minute (MI) or Month (MO)
void MCase(){
#if defined(DEBUG)
  Serial.println("MCase"); 
 #endif 
   switch (commandArray[1])
  {
     case 'I': // Minute
       MICase();
       break; 
     case 'O': // Month
       MOCase();
       break; 
      default:
       Serial.println("MCase Bad commandArray[1].");
       break;   
  }
}

// Minute
void MICase(){
#if defined(DEBUG)  
  Serial.println("MICase");
#endif  
}

// Month
void MOCase(){
#if defined(DEBUG)  
  Serial.println("MOCase"); 
#endif 
}


// Second
void SCase(){
#if defined(DEBUG)  
  Serial.println("SCase"); 
#endif 
}

// Set Alarm beep type
void TCase(){
#if defined(DEBUG)
  //Serial.println("TCase"); 
#endif 
  if( (commandArray[1] == 'i') && \
    (commandArray[2] == 'm') && \
    (commandArray[3] == 'e') ) show_time();
  else  Serial.println("TCase Bad commandArray[].");  
}

 // Year
void YCase(){
#if defined(DEBUG)
  Serial.println("YCase");
#endif  
}

void NCase(){
#if defined(DEBUG)
  Serial.println("NCase"); 
#endif
  showNow();
}

void PCase(){
#if defined(DEBUG)
  Serial.println("PCase");
#endif 
  setDateTimeToPC();
}

void LCase(){
#if defined(DEBUG)
  Serial.println("LCase");
#endif 
 Serial.print("alarm1 = ");
 Serial.println(alarm1.datetime.unixtime(),DEC);
 Serial.print("alarm2 = ");
 Serial.println(alarm2.datetime.unixtime(),DEC);
 Serial.print("alarm3 = ");
 Serial.println(alarm3.datetime.unixtime(),DEC); 
 Serial.print("alarm4 = ");
 Serial.println(alarm4.datetime.unixtime(),DEC);  
 Serial.print("alarm5 = ");
 Serial.println(alarm5.datetime.unixtime(),DEC); 
 
 DateTime now = RTC.now();
 Serial.print("now = ");
 Serial.println(now.unixtime(),DEC); 
 if (now.unixtime() >= alarm1.datetime.unixtime()) Serial.println("Alarm1 elapsed");
 else Serial.println("Alarm1 pending");
 if (now.unixtime() >= alarm2.datetime.unixtime()) Serial.println("Alarm2 elapsed");
 else Serial.println("Alarm2 pending");
 if (now.unixtime() >= alarm3.datetime.unixtime()) Serial.println("Alarm3 elapsed");
 else Serial.println("Alarm3 pending");
 if (now.unixtime() >= alarm4.datetime.unixtime()) Serial.println("Alarm4 elapsed");
 else Serial.println("Alarm4 pending");
 if (now.unixtime() >= alarm5.datetime.unixtime()) Serial.println("Alarm5 elapsed");
 else Serial.println("Alarm5 pending");
 
}

void UCase(){
#if defined(DEBUG)
 Serial.println("Ucase");
 Serial.print("Unset Alarm");
 Serial.println(commandArray[1]);
#endif 
  switch (commandArray[1])
  {
     case '1': // Alarm1
       unset_alarm(alarm1.datetime);
       alarm1.is_set = 0;
       alarm1.is_tripped = 0;
       break; 
     case '2': // Alarm2
       unset_alarm(alarm2.datetime);
       alarm2.is_set = 0;
       alarm2.is_tripped = 0;
       break;
      case '3': // Alarm3
       unset_alarm(alarm3.datetime);
       alarm3.is_set = 0;
       alarm3.is_tripped = 0;
       break;     
      case '4': // Alarm4
       unset_alarm(alarm4.datetime);
       alarm4.is_set = 0;
       alarm4.is_tripped = 0;
       break;      
      case '5': // Alarm
       unset_alarm(alarm5.datetime);
       alarm5.is_set = 0;
       alarm5.is_tripped = 0;
       break;            
     default:
       Serial.println("UCase Bad commandArray[1].");
       break;   
  }  
}

