//
// Process_Alarms module for the time keeping functions
//
// Most of the DS1307 timekeeping items and concepts were taken from 
// RTCLib provided by Jean-Claude Wippler of www.jeelabs.com
//

// Initialzie the I2C communications and the DS1307 Real Time Clock
void initializeDS1307RTC(){
  Wire.begin(); // initialize the I2C communications
  RTC.begin(); // initialize the DS1307 real time clock
}

void initializeAlarms(){ 
  
  for(int i = 0; i < 5; i++){
    read_alarm(i);
  }
  
  read_calibration();
}

void go_calibration(){
    // Get the current date time
    DateTime now = RTC.now();  
    uint32_t myNow = now.unixtime();
    
    // Check if it is time to run a calibration
    if( myNow >=  cal_start_time + calibration){
      if(addsubtract){  // 1 indicates subract a second    
        RTC.adjust(DateTime(now.unixtime() - 1));
        Serial.println("Calibration subtracted a second");
      }
      else{ // 0 indicates add a second
        RTC.adjust(DateTime(now.unixtime() + 2));
        Serial.println("Calibration added a second");
      }     
      cal_start_time += calibration;
    }  
}

void go_alarm1(){
#if defined(DEBUG)   
  //Serial.println("go_alarm1");
#endif 

  if(alarm1.is_set){
    DateTime now = RTC.now();  
    if(now.unixtime() >= alarm1.datetime.unixtime()){
     //Perform the action called for
      //Serial.println("Alarm1 tripped!");  
      alarm1.is_tripped = 1; 
    }    
    if(alarm1.type == 0){ 
       // do nothing
     }
     else if(alarm1.type == 1){// if daily, add 24 hrs to the alarm and leave set
       /*Serial.print("Old alarm date time = ");
       show_date("alarm1",alarm1.datetime); 
       // one day later
       DateTime temp(alarm1.datetime.unixtime() + 86400L); 
       alarm1.datetime = temp;
       Serial.print("New alarm date time = ");
       show_date("alarm1",alarm1.datetime);*/
     }
     else Serial.print("alarm1_type neither");
  } 
  
  if(alarm1.is_tripped)
  {
    // if the button state is not pressed, run the alarm
    if(!buttonState) warble3_beep = 1; 
    else {
      alarm1.is_tripped = 0; // turn this alarm off
      warble3_beep = 0; // turn the beep off
      if(alarm1.type == 0){ // if one shot, unset the alarm
       if(buttonState)unset_alarm(alarm1.datetime);
     }     
    }
  }  
 
  /*Serial.print(".");
  delay(500);
  DateTime now = RTC.now();  
  if(now.unixtime() <= alarm1.unixtime()){
    Serial.println("Alarm1 just went off!");   
 
  }*/
  
/*  Serial.println("1");
  DateTime now = RTC.now();  
  Serial.println("2");
  THIS KILLS IT if(now.unixtime() <= alarm1.unixtime()){
    Serial.println("Alarm1 just went off!");   
     alarm1_set = 0; 
  }
  Serial.println("3");  
*/
}

void go_alarm2(){
  
}
void go_alarm3(){
  
}
void go_alarm4(){
  
}
void go_alarm5(){
  
}

void setAlarm(DateTime& dt){
#if defined(DEBUG)
 Serial.println("setAlarm"); 
#endif 
  dt.set_year(( (commandArray[2]-48) * 10) + (commandArray[3]-48)); 
  dt.set_month(( (commandArray[4]-48) * 10) + (commandArray[5]-48));
  dt.set_day(( (commandArray[6]-48) * 10) + (commandArray[7]-48)); 
  dt.set_hour(( (commandArray[8]-48) * 10) + (commandArray[9]-48)); 
  dt.set_minute(( (commandArray[10]-48) * 10) + (commandArray[11]-48)); 
  dt.set_second(( (commandArray[12]-48) * 10) + (commandArray[13]-48)); 
  //Serial.print("in setAlarm, unixtime = ");
  //Serial.println(dt.unixtime());
}

void unset_alarm(DateTime& dt){
#if defined(DEBUG)
 Serial.println("unset_alarm"); 
#endif 
  dt.set_year(0); 
  dt.set_month(0);
  dt.set_day(0); 
  dt.set_hour(0); 
  dt.set_minute(0); 
  dt.set_second(0); 
}

// Note - this is educational novice friendly code and does not use
// some more effiecient, but also more difficult C constructs.
void write_alarm(int alarm_number, struct alarm alar){//DateTime alarm, int alarm_type, int alarm_set, int alarm_tripped){
  uint8_t val1,val2,val3,val4;
  uint32_t unix_value = (uint32_t)alar.datetime.unixtime();
  val1 = (uint8_t)(unix_value & 0x000000FF);
  val2 = (uint8_t)((unix_value & 0x0000FF00)>>8); 
  val3 = (uint8_t)((unix_value & 0x00FF0000)>>16); 
  val4 = (uint8_t)((unix_value & 0xFF000000)>>24); 

/*#if defined(DEBUG)
  Serial.print("For alarm #: ");
  Serial.println(alarm_number);
  Serial.print("For datetime = ");
  Serial.println(unix_value);
  Serial.print("val1 = ");
  Serial.println(val1);
  Serial.print("val2 = ");
  Serial.println(val2);
  Serial.print("val3 = ");
  Serial.println(val3);  
  Serial.print("val4 = ");
  Serial.println(val4); 
  Serial.print("type = ");
  Serial.println(alar.type); 
  Serial.print("set = ");
  Serial.println(alar.is_set);   
  Serial.print("tripped = ");
  Serial.println(alar.is_tripped);    
#endif */
  
  switch (alarm_number)
  {
     case 1: // write alarm 1
       EEPROM.write(A1A,val1);
       EEPROM.write(A1A+1,val2);
       EEPROM.write(A1A+2,val3);
       EEPROM.write(A1A+3,val4);
       EEPROM.write(A1A+4,alar.type);
       EEPROM.write(A1A+5,alar.is_set);
       EEPROM.write(A1A+6,alar.is_tripped);     
       break; 
     /*case 2: // write alarm 2
       EEPROM.write(A2A,val1);
       EEPROM.write(A2A+1,val2);
       EEPROM.write(A2A+2,val3);
       EEPROM.write(A2A+3,val4);
       EEPROM.write(A2A+4,alarm_type);
       EEPROM.write(A2A+5,alarm_set);
       EEPROM.write(A2A+6,alarm_tripped); 
       break;   
     case 3: // write alarm 3
       EEPROM.write(A3A,val1);
       EEPROM.write(A3A+1,val2);
       EEPROM.write(A3A+2,val3);
       EEPROM.write(A3A+3,val4);
       EEPROM.write(A3A+4,alarm_type);
       EEPROM.write(A3A+5,alarm_set);
       EEPROM.write(A3A+6,alarm_tripped);
       break; 
     case 4: // write alarm 4
       EEPROM.write(A4A,val1);
       EEPROM.write(A4A+1,val2);
       EEPROM.write(A4A+2,val3);
       EEPROM.write(A4A+3,val4);
       EEPROM.write(A4A+4,alarm_type);
       EEPROM.write(A4A+5,alarm_set);
       EEPROM.write(A4A+6,alarm_tripped);      
       break;      
     case 5: // write alarm 5
       EEPROM.write(A5A,val1);
       EEPROM.write(A5A+1,val2);
       EEPROM.write(A5A+2,val3);
       EEPROM.write(A5A+3,val4);
       EEPROM.write(A5A+4,alarm_type);
       EEPROM.write(A5A+5,alarm_set);
       EEPROM.write(A5A+6,alarm_tripped);       
       break;  */   
     default:
       Serial.print("ERROR: write_alarm() bad alarm_number ");
       break; 
  }       
}

void read_alarm(int alarm_number){
#if defined(DEBUG)   
  Serial.println("read_alarm"); 
#endif  
  switch (alarm_number)
  {
     case 1: // read alarm 1
       alarm1.datetime = (uint32_t)EEPROM.read(A1A) + \
                               ((uint32_t)EEPROM.read(A1A+1)<<8) + \
                               ((uint32_t)EEPROM.read(A1A+2)<<16) + \
                               ((uint32_t)EEPROM.read(A1A+3)<<24); 
       alarm1.type = A1A+4;
       alarm1.is_set = A1A+5;
       alarm1.is_tripped = A1A+6;     
       break; 
     /*case 2: // read alarm 2
       alarm2.datetime = (uint32_t)EEPROM.read(A2A) + \
                               ((uint32_t)EEPROM.read(A2A+1)<<8) + \
                               ((uint32_t)EEPROM.read(A2A+2)<<16) + \
                               ((uint32_t)EEPROM.read(A2A+3)<<24); 
       alarm2_type = A2A+4;
       alarm2_set = A2A+5;
       alarm2_tripped = A2A+6;  
       break; 
      case 3: // read alarm 3
       alarm3.datetime = (uint32_t)EEPROM.read(A3A) + \
                               ((uint32_t)EEPROM.read(A3A+1)<<8) + \
                               ((uint32_t)EEPROM.read(A3A+2)<<16) + \
                               ((uint32_t)EEPROM.read(A3A+3)<<24); 
       alarm3_type = A3A+4;
       alarm3_set = A3A+5;
       alarm3_tripped = A3A+6;  
       break;    
      case 4: // read alarm 4
       alarm4.datetime = (uint32_t)EEPROM.read(A4A) + \
                               ((uint32_t)EEPROM.read(A4A+1)<<8) + \
                               ((uint32_t)EEPROM.read(A4A+2)<<16) + \
                               ((uint32_t)EEPROM.read(A4A+3)<<24); 
       alarm4_type = A4A+4;
       alarm4_set = A4A+5;
       alarm4_tripped = A4A+6;  
       break;    
      case 5: // read alarm 5
       alarm5.datetime = (uint32_t)EEPROM.read(A5A) + \
                               ((uint32_t)EEPROM.read(A5A+1)<<8) + \
                               ((uint32_t)EEPROM.read(A5A+2)<<16) + \
                               ((uint32_t)EEPROM.read(A5A+3)<<24); 
       alarm5_type = A5A+4;
       alarm5_set = A5A+5;
       alarm5_tripped = A5A+6;  
       break;  */      
     default:
       Serial.print("ERROR: read_alarm() bad alarm_number ");

  }   
}

void write_calibration(){
  uint8_t val1,val2,val3,val4;

  val1 = (uint8_t)(calibration & 0x000000FF);
  val2 = (uint8_t)((calibration & 0x0000FF00)>>8); 
  val3 = (uint8_t)((calibration & 0x00FF0000)>>16); 
  val4 = (uint8_t)((calibration & 0xFF000000)>>24); 
  EEPROM.write(CAL_EEPROM_ADDRESS,val1);
  EEPROM.write(CAL_EEPROM_ADDRESS+1,val2);
  EEPROM.write(CAL_EEPROM_ADDRESS+2,val3);
  EEPROM.write(CAL_EEPROM_ADDRESS+3,val4);
 
  val1 = (uint8_t)(cal_start_time & 0x000000FF);
  val2 = (uint8_t)((cal_start_time & 0x0000FF00)>>8); 
  val3 = (uint8_t)((cal_start_time & 0x00FF0000)>>16); 
  val4 = (uint8_t)((cal_start_time & 0xFF000000)>>24); 
  EEPROM.write(CAL_EEPROM_ADDRESS+4,val1);
  EEPROM.write(CAL_EEPROM_ADDRESS+5,val2);
  EEPROM.write(CAL_EEPROM_ADDRESS+6,val3);
  EEPROM.write(CAL_EEPROM_ADDRESS+7,val4); 
  
  EEPROM.write(CAL_EEPROM_ADDRESS+8,cal_is_set);
  EEPROM.write(CAL_EEPROM_ADDRESS+9,addsubtract);        
}

void read_calibration(){
       calibration = (uint32_t)EEPROM.read(CAL_EEPROM_ADDRESS) + \
                               ((uint32_t)EEPROM.read(CAL_EEPROM_ADDRESS+1)<<8) + \
                               ((uint32_t)EEPROM.read(CAL_EEPROM_ADDRESS+2)<<16) + \
                               ((uint32_t)EEPROM.read(CAL_EEPROM_ADDRESS+3)<<24);
       cal_start_time = (uint32_t)EEPROM.read(CAL_EEPROM_ADDRESS+4) + \
                               ((uint32_t)EEPROM.read(CAL_EEPROM_ADDRESS+5)<<8) + \
                               ((uint32_t)EEPROM.read(CAL_EEPROM_ADDRESS+6)<<16) + \
                               ((uint32_t)EEPROM.read(CAL_EEPROM_ADDRESS+7)<<24);                              
                               
       cal_is_set = EEPROM.read(CAL_EEPROM_ADDRESS+8);
       addsubtract = EEPROM.read(CAL_EEPROM_ADDRESS+9);
}


